import { generateMessageId, GunGraphWireConnector } from '@notabug/chaingun';
import lmdb from 'node-lmdb';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var DEFAULT_CONFIG = {
    path: 'lmdb'
};
var GunLmdbClient = /** @class */ (function () {
    function GunLmdbClient(lmdbConfig) {
        if (lmdbConfig === void 0) { lmdbConfig = DEFAULT_CONFIG; }
        this.lmdb = lmdb;
        this.env = new lmdb.Env();
        this.env.open(lmdbConfig);
        this.dbi = this.env.openDbi({
            name: 'gun-nodes',
            create: true
        });
    }
    GunLmdbClient.prototype.get = function (soul) {
        if (!soul)
            return null;
        var txn = this.env.beginTxn({ readOnly: true });
        try {
            var data = this.deserialize(txn.getStringUnsafe(this.dbi, soul));
            txn.commit();
            return data;
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.getRaw = function (soul) {
        if (!soul)
            return null;
        var txn = this.env.beginTxn({ readOnly: true });
        try {
            var data = txn.getString(this.dbi, soul);
            txn.commit();
            return data || '';
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.serialize = function (node) {
        return JSON.stringify(node);
    };
    GunLmdbClient.prototype.deserialize = function (data) {
        return JSON.parse(data);
    };
    GunLmdbClient.prototype.writeNode = function (soul, nodeData) {
        if (!soul)
            return;
        var txn = this.env.beginTxn();
        var nodeDataMeta = (nodeData && nodeData['_']) || {};
        var nodeDataState = nodeDataMeta['>'] || {};
        try {
            var existingData = txn.getStringUnsafe(this.dbi, soul);
            var node = this.deserialize(existingData) || {};
            var meta = (node['_'] = node['_'] || { '#': soul, '>': {} });
            var state = (meta['>'] = meta['>'] || {});
            for (var key in nodeData) {
                if (key === '_' || !(key in nodeDataState))
                    continue;
                node[key] = nodeData[key];
                state[key] = nodeDataState[key];
            }
            txn.putString(this.dbi, soul, this.serialize(node));
            txn.commit();
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.write = function (put) {
        if (!put)
            return;
        for (var soul in put)
            this.writeNode(soul, put[soul]);
    };
    GunLmdbClient.prototype.close = function () {
        this.dbi.close();
        this.env.close();
    };
    return GunLmdbClient;
}());

var NOOP = function () { };
var LmdbGraphConnector = /** @class */ (function (_super) {
    __extends(LmdbGraphConnector, _super);
    function LmdbGraphConnector(lmdbOpts) {
        if (lmdbOpts === void 0) { lmdbOpts = DEFAULT_CONFIG; }
        var _this = _super.call(this, 'LmdbGraphConnector') || this;
        _this._client = new GunLmdbClient(lmdbOpts);
        return _this;
    }
    LmdbGraphConnector.prototype.get = function (_a) {
        var _b;
        var soul = _a.soul, cb = _a.cb, _c = _a.msgId, msgId = _c === void 0 ? '' : _c, _d = _a.key // TODO
        ;
        var node = this._client.get(soul);
        var put = node
            ? (_b = {},
                _b[soul] = node,
                _b) : null;
        var msg = { '#': generateMessageId(), put: put };
        if (msgId)
            msg['@'] = msgId;
        if (cb)
            cb(msg);
        return NOOP;
    };
    LmdbGraphConnector.prototype.put = function (_a) {
        var graph = _a.graph, _b = _a.msgId, msgId = _b === void 0 ? '' : _b, cb = _a.cb;
        var id = generateMessageId();
        try {
            this._client.write(graph);
            if (cb) {
                cb({
                    '#': id,
                    '@': msgId,
                    ok: true,
                    err: null
                });
            }
        }
        catch (err) {
            if (cb) {
                cb({
                    '#': id,
                    '@': msgId,
                    ok: false,
                    err: err
                });
            }
            else {
                console.warn(err.stack || err);
            }
        }
        return NOOP;
    };
    return LmdbGraphConnector;
}(GunGraphWireConnector));

export default LmdbGraphConnector;
//# sourceMappingURL=chaingun-lmdb.es5.js.map
