"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var node_lmdb_1 = require("node-lmdb");
exports.DEFAULT_CONFIG = {
    path: 'lmdb'
};
var GunLmdbClient = /** @class */ (function () {
    function GunLmdbClient(lmdbConfig) {
        if (lmdbConfig === void 0) { lmdbConfig = exports.DEFAULT_CONFIG; }
        this.lmdb = node_lmdb_1.default;
        this.env = new node_lmdb_1.default.Env();
        this.env.open(lmdbConfig);
        this.dbi = this.env.openDbi({
            name: 'gun-nodes',
            create: true
        });
    }
    GunLmdbClient.prototype.get = function (soul) {
        if (!soul)
            return null;
        var txn = this.env.beginTxn({ readOnly: true });
        try {
            var data = this.deserialize(txn.getStringUnsafe(this.dbi, soul));
            txn.commit();
            return data;
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.getRaw = function (soul) {
        if (!soul)
            return null;
        var txn = this.env.beginTxn({ readOnly: true });
        try {
            var data = txn.getString(this.dbi, soul);
            txn.commit();
            return data || '';
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.serialize = function (node) {
        return JSON.stringify(node);
    };
    GunLmdbClient.prototype.deserialize = function (data) {
        return JSON.parse(data);
    };
    GunLmdbClient.prototype.writeNode = function (soul, nodeData) {
        if (!soul)
            return;
        var txn = this.env.beginTxn();
        var nodeDataMeta = (nodeData && nodeData['_']) || {};
        var nodeDataState = nodeDataMeta['>'] || {};
        try {
            var existingData = txn.getStringUnsafe(this.dbi, soul);
            var node = this.deserialize(existingData) || {};
            var meta = (node['_'] = node['_'] || { '#': soul, '>': {} });
            var state = (meta['>'] = meta['>'] || {});
            for (var key in nodeData) {
                if (key === '_' || !(key in nodeDataState))
                    continue;
                node[key] = nodeData[key];
                state[key] = nodeDataState[key];
            }
            txn.putString(this.dbi, soul, this.serialize(node));
            txn.commit();
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.write = function (put) {
        if (!put)
            return;
        for (var soul in put)
            this.writeNode(soul, put[soul]);
    };
    GunLmdbClient.prototype.close = function () {
        this.dbi.close();
        this.env.close();
    };
    return GunLmdbClient;
}());
exports.GunLmdbClient = GunLmdbClient;
//# sourceMappingURL=GunLmdbClient.js.map